@@ -78,7 +78,7 @@ void ICalFormatTest::testDeserializeSerialize()
     QVERIFY(occurrence->hasRecurrenceId());
     QCOMPARE(occurrence->recurrenceId(), start.addDays(1));
 
-    const QString serialization = format.toString(calendar, QString());
+    const QString serialization = format.toString(calendar);
     QVERIFY(!serialization.isEmpty());
     MemoryCalendar::Ptr check = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
     QVERIFY(format.fromString(check, serialization));
@@ -101,12 +101,12 @@ void ICalFormatTest::testCharsets()
     event->setAllDay(true);
 
     // ü
-    const QChar latin1_umlaut[] = {0xFC, QLatin1Char('\0')};
+    const QChar latin1_umlaut[] = {(QChar)0xFC, QLatin1Char('\0')};
     event->setSummary(QString(latin1_umlaut));
 
     // Test if toString( Incidence ) didn't mess charsets
     const QString serialized = format.toString(event.staticCast<Incidence>());
-    const QChar utf_umlaut[] = {0xC3, 0XBC, QLatin1Char('\0')};
+    const QChar utf_umlaut[] = {(QChar)0xC3, (QChar)0XBC, QLatin1Char('\0')};
     QVERIFY(serialized.toUtf8().contains(QString(utf_umlaut).toLatin1().constData()));
     QVERIFY(!serialized.toUtf8().contains(QString(latin1_umlaut).toLatin1().constData()));
     QVERIFY(serialized.toLatin1().contains(QString(latin1_umlaut).toLatin1().constData()));
@@ -308,7 +308,7 @@ void ICalFormatTest::testRDate()
     QCOMPARE(it.occurrenceStartDate(), ev4);
     QCOMPARE(it.occurrenceEndDate(), ev4.addSecs(7200));
 
-    const QStringList output = format.toString(calendar, QString()).split(QString::fromLatin1("\r\n"));
+    const QStringList output = format.toString(calendar).split(QString::fromLatin1("\r\n"));
     QVERIFY(output.contains(QString::fromLatin1("RDATE;VALUE=PERIOD:20210630T100000Z/20210630T110000Z")));
     QVERIFY(output.contains(QString::fromLatin1("RDATE;VALUE=PERIOD:20210825T100000Z/20210825T110000Z")));
     QVERIFY(output.contains(QString::fromLatin1("RDATE;VALUE=PERIOD:20211027T100000Z/20211027T110000Z")));
@@ -350,3 +350,181 @@ void ICalFormatTest::testDateTime()
     QVERIFY(event);
     QCOMPARE(dtStart, event->dtStart());
 }
+
+/**
+ * If an instance does not have a UID, one will be created for it.
+ */
+void ICalFormatTest::testUidGeneration()
+{
+    const QString serializedCalendar = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "DTSTAMP:20201103T161340Z\n"
+        "SUMMARY:test\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    ICalFormat format;
+    QVERIFY(format.fromString(calendar, serializedCalendar));
+    const auto events = calendar->events();
+    QCOMPARE(events.count(), 1);
+    const auto event = events[0];
+    QVERIFY( ! event->uid().isEmpty());
+}
+
+/**
+ * Generated UIDs do not depend on the order of properties.
+ */
+void ICalFormatTest::testUidGenerationStability()
+{
+    ICalFormat format;
+
+    const QString serializedCalendar1 = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "DTSTAMP:20201103T161340Z\n"
+        "SUMMARY:test\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar1 = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(calendar1, serializedCalendar1));
+    const auto events1 = calendar1->events();
+    QCOMPARE(events1.count(), 1);
+
+    const QString serializedCalendar2 = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "SUMMARY:test\n"
+        "DTSTAMP:20201103T161340Z\n"    // Reordered.
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar2 = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(calendar2, serializedCalendar2));
+    const auto events2 = calendar2->events();
+    QCOMPARE(events2.count(), 1);
+
+    const auto event1 = events1[0];
+    const auto event2 = events2[0];
+    QCOMPARE(event1->uid(), event2->uid());
+}
+
+/**
+ * Generated UIDs depend on property names and values.
+ */
+void ICalFormatTest::testUidGenerationUniqueness()
+{
+    ICalFormat format;
+
+    const QString serializedCalendar1 = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "DTSTAMP:20201103T161340Z\n"
+        "SUMMARY:test\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar1 = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(calendar1, serializedCalendar1));
+    const auto events1 = calendar1->events();
+    QCOMPARE(events1.count(), 1);
+
+    const QString serializedCalendar2 = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "DTSTART:20201103T161340Z\n"    // Property name change.
+        "SUMMARY:test\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar2 = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(calendar2, serializedCalendar2));
+    const auto events2 = calendar2->events();
+    QCOMPARE(events2.count(), 1);
+
+    const QString serializedCalendar3 = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "DTSTAMP:20201103T161341Z\n"    // Property value changed.
+        "SUMMARY:test\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR");
+    auto calendar3 = MemoryCalendar::Ptr(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(calendar3, serializedCalendar3));
+    const auto events3 = calendar3->events();
+    QCOMPARE(events3.count(), 1);
+
+    const auto event1 = events1[0];
+    const auto event2 = events2[0];
+    const auto event3 = events3[0];
+    QVERIFY(event1->uid() != event2->uid());
+    QVERIFY(event1->uid() != event3->uid());
+    QVERIFY(event2->uid() != event3->uid());
+}
+
+void ICalFormatTest::testIcalFormat()
+{
+    ICalFormat format;
+    auto duration = format.durationFromString(QStringLiteral("PT2H"));
+    QVERIFY(!duration.isNull());
+    QCOMPARE(duration.asSeconds(), 7200);
+    QCOMPARE(format.toString(duration), QLatin1String("PT2H"));
+}
+
+void ICalFormatTest::testNonTextCustomProperties()
+{
+    const auto input = QLatin1String(
+        "BEGIN:VCALENDAR\n"
+        "VERSION:2.0\n"
+        "BEGIN:VEVENT\n"
+        "X-APPLE-TRAVEL-START;ROUTING=CAR;VALUE=URI;X-ADDRESS=Bingerdenallee 1\\n\n"
+        " 6921 JN Duiven\\nNederland;X-TITLE=Home:\n"
+        "X-APPLE-TRAVEL-DURATION;VALUE=DURATION:PT45M\n"
+        "X-APPLE-STRUCTURED-LOCATION;VALUE=URI;X-ADDRESS=Olympus 1\\n3524 WB Utre\n"
+        " cht\\nThe Netherlands;X-APPLE-RADIUS=49.91307222863458;X-TITLE=Olympus 1\n"
+        " :geo:52.063921,5.128511\n"
+        "BEGIN:VALARM\n"
+        "TRIGGER;X-APPLE-RELATED-TRAVEL=-PT30M:-PT1H15M\n"
+        "END:VALARM\n"
+        "END:VEVENT\n"
+        "END:VCALENDAR\n");
+    ICalFormat format;
+    MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
+    QVERIFY(format.fromString(cal, input));
+    const auto events = cal->events();
+    QCOMPARE(events.size(), 1);
+
+    const auto event = events[0];
+    QCOMPARE(event->nonKDECustomProperty("X-APPLE-TRAVEL-DURATION"), QLatin1String("PT45M"));
+    QCOMPARE(event->nonKDECustomProperty("X-APPLE-TRAVEL-START"), QString());
+    QCOMPARE(event->nonKDECustomProperty("X-APPLE-STRUCTURED-LOCATION"), QLatin1String("geo:52.063921,5.128511"));
+}
+
+void ICalFormatTest::testAllDaySchedulingMessage()
+{
+    auto event = KCalendarCore::Event::Ptr::create();
+    event->setSummary(QStringLiteral("All Day Event"));
+    event->setDtStart(QDateTime(QDate(2023, 10, 13), QTime(0, 0, 0), QTimeZone("Europe/Prague")));
+    event->setDtEnd(QDateTime(QDate(2023, 10, 15), QTime(0, 0, 0), QTimeZone("Europe/Prague")));
+    event->setOrganizer(Person(QStringLiteral("Dan"), QStringLiteral("dvratil@example.com")));
+    event->addAttendee(Attendee(QStringLiteral("Konqi"), QStringLiteral("konqi@example.com")));
+    event->setAllDay(true);
+
+    ICalFormat format;
+    auto calendar = MemoryCalendar::Ptr::create(QTimeZone::utc());
+    const auto itipString = format.createScheduleMessage(event, KCalendarCore::iTIPRequest);
+    QVERIFY(!itipString.isEmpty());
+
+    auto scheduleMsg = format.parseScheduleMessage(calendar, itipString);
+    QVERIFY(scheduleMsg->error().isEmpty());
+
+    auto parsedEvent = scheduleMsg->event().staticCast<KCalendarCore::Event>();
+    QVERIFY(parsedEvent);
+    QCOMPARE(parsedEvent->dtStart().date(), event->dtStart().date());
+    QCOMPARE(parsedEvent->dtEnd().date(), event->dtEnd().date());
+}
+
+#include "moc_testicalformat.cpp"
