@@ -18,35 +18,6 @@ QTEST_MAIN(MemoryCalendarTest)
 
 using namespace KCalendarCore;
 
-void MemoryCalendarTest::testClose()
-{
-    MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
-
-    Event::Ptr event1(new Event);
-    cal->addIncidence(event1);
-
-    const QString nbuid = QString::fromLatin1("test-notebook");
-    QVERIFY(cal->addNotebook(nbuid, true));
-
-    Event::Ptr event2(new Event);
-    cal->addIncidence(event2);
-    cal->setNotebook(event2, nbuid);
-
-    QCOMPARE(cal->incidences().count(), 2);
-    QVERIFY(cal->instance(event1->instanceIdentifier()));
-    QVERIFY(cal->instance(event2->instanceIdentifier()));
-    QVERIFY(cal->event(event1->uid()));
-    QVERIFY(cal->event(event2->uid()));
-    QCOMPARE(cal->incidences(nbuid).count(), 1);
-
-    cal->close();
-
-    QVERIFY(!cal->event(event1->uid()));
-    QVERIFY(!cal->event(event2->uid()));
-    QVERIFY(cal->incidences().isEmpty());
-    QVERIFY(cal->incidences(nbuid).isEmpty());
-}
-
 void MemoryCalendarTest::testValidity()
 {
     MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
@@ -54,7 +25,6 @@ void MemoryCalendarTest::testValidity()
     QVERIFY(cal->productId() == QLatin1String("fredware calendar"));
     QVERIFY(cal->timeZoneId() == QByteArrayLiteral("UTC"));
     QVERIFY(cal->timeZone() == QTimeZone::utc());
-    cal->close();
 }
 
 void MemoryCalendarTest::testInvalidTimeZone()
@@ -93,7 +63,6 @@ void MemoryCalendarTest::testEvents()
 
     FileStorage store(cal, QStringLiteral("foo.ics"));
     QVERIFY(store.save());
-    cal->close();
     QFile::remove(QStringLiteral("foo.ics"));
 }
 
@@ -146,14 +115,12 @@ void MemoryCalendarTest::testIncidences()
 
     FileStorage store(cal, QStringLiteral("foo.ics"));
     QVERIFY(store.save());
-    cal->close();
 
     QVERIFY(store.load());
     Todo::Ptr todo = cal->incidence(QStringLiteral("4")).staticCast<Todo>();
     QVERIFY(todo->uid() == QLatin1Char('4'));
     QVERIFY(todo->summaryIsRich());
     QVERIFY(todo->locationIsRich());
-    cal->close();
     QFile::remove(QStringLiteral("foo.ics"));
 }
 
@@ -196,7 +163,6 @@ void MemoryCalendarTest::testRelationsCrash()
         }
       }
     */
-    cal->close();
 }
 
 void MemoryCalendarTest::testRecurrenceExceptions()
@@ -327,27 +293,6 @@ void MemoryCalendarTest::testRawEventsForDate()
     QVERIFY(cal->addEvent(event));
     QCOMPARE(cal->rawEventsForDate(QDate(2019, 10, 31)).count(), 1);
     QCOMPARE(cal->rawEventsForDate(QDate(2019, 11, 1)).count(), 1);
-
-    cal->close();
-}
-
-void MemoryCalendarTest::testVisibility()
-{
-    MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
-    const QString notebook = QLatin1String("Notebook");
-
-    QVERIFY(cal->addNotebook(notebook, true));
-    QVERIFY(cal->isVisible(notebook));
-
-    Event::Ptr event = Event::Ptr(new Event());
-    QVERIFY(cal->addIncidence(event));
-    QVERIFY(cal->setNotebook(event, notebook));
-
-    QVERIFY(cal->isVisible(event));
-
-    QVERIFY(cal->updateNotebook(notebook, false));
-    QVERIFY(!cal->isVisible(notebook));
-    QVERIFY(!cal->isVisible(event));
 }
 
 class TestCalendarObserver: public Calendar::CalendarObserver
@@ -379,42 +324,6 @@ private:
     Calendar::Ptr mCalendar;
 };
 
-void MemoryCalendarTest::testNotebookChange()
-{
-    MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
-    TestCalendarObserver observer(cal);
-    const QString notebook1 = QLatin1String("Notebook1");
-    const QString notebook2 = QLatin1String("Notebook2");
-
-    QVERIFY(cal->addNotebook(notebook1, true));
-    QVERIFY(cal->addNotebook(notebook2, true));
-
-    Event::Ptr event = Event::Ptr(new Event());
-    event->setDtStart(QDateTime(QDate(2022, 3, 23), QTime(10, 12), Qt::UTC));
-    event->recurrence()->setDaily(1);
-    Incidence::Ptr exception = Calendar::createException(event, event->dtStart().addDays(3));
-    QVERIFY(exception);
-    exception->setDtStart(exception->recurrenceId().addSecs(1800));
-    QVERIFY(cal->addIncidence(event));
-    QVERIFY(cal->addIncidence(exception));
-    QVERIFY(cal->notebook(event).isEmpty());
-    QVERIFY(cal->notebook(exception).isEmpty());
-
-    QVERIFY(observer.mUpdated.isEmpty());
-    QVERIFY(cal->setNotebook(event, notebook1));
-    QCOMPARE(cal->notebook(event), notebook1);
-    QCOMPARE(cal->notebook(exception), notebook1);
-    QVERIFY(observer.hasIncidenceChanged(event));
-    QVERIFY(observer.hasIncidenceChanged(exception));
-
-    observer.mUpdated.clear();
-    QVERIFY(cal->setNotebook(event, notebook2));
-    QCOMPARE(cal->notebook(event), notebook2);
-    QCOMPARE(cal->notebook(exception), notebook2);
-    QVERIFY(observer.hasIncidenceChanged(event));
-    QVERIFY(observer.hasIncidenceChanged(exception));
-}
-
 void MemoryCalendarTest::testRawEvents()
 {
     MemoryCalendar::Ptr cal(new MemoryCalendar(QTimeZone::utc()));
@@ -458,8 +367,6 @@ void MemoryCalendarTest::testRawEvents()
     QCOMPARE(cal->rawEvents(QDate(2020, 10, 13), QDate(), QTimeZone(), true).count(), 0);
     QCOMPARE(cal->rawEvents(QDate(2020, 10, 12), QDate(), QTimeZone(), true).count(), 0);
     QCOMPARE(cal->rawEvents(QDate(2020, 10, 11), QDate(), QTimeZone(), true).count(), 1);
-
-    cal->close();
 }
 
 void MemoryCalendarTest::testDeleteIncidence()
