@@ -128,24 +128,6 @@ bool IncidencePrivate::validStatus(Incidence::Status status)
 }
 //@endcond
 
-#if KCALENDARCORE_BUILD_DEPRECATED_SINCE(5, 91)
-Incidence::Incidence()
-    : IncidenceBase(new IncidencePrivate)
-{
-    recreate();
-    resetDirtyFields();
-}
-
-Incidence::Incidence(const Incidence &i)
-    : IncidenceBase(i, new IncidencePrivate(*(i.d_func())))
-    , Recurrence::RecurrenceObserver()
-{
-    Q_D(Incidence);
-    d->init(this, *(i.d_func()));
-    resetDirtyFields();
-}
-#endif
-
 Incidence::Incidence(IncidencePrivate *p)
     : IncidenceBase(p)
 {
@@ -1043,25 +1025,7 @@ bool Incidence::hasGeo() const
     // For internal consistency, return false if either coordinate is invalid.
     return d->mGeoLatitude != INVALID_LATLON && d->mGeoLongitude != INVALID_LATLON;
 }
-#if KCALENDARCORE_BUILD_DEPRECATED_SINCE(5, 89)
-void Incidence::setHasGeo(bool hasGeo)
-{
-    if (mReadOnly) {
-        return;
-    }
 
-    Q_D(Incidence);
-    if (!hasGeo) {
-        update();
-        d->mGeoLatitude = INVALID_LATLON;
-        d->mGeoLongitude = INVALID_LATLON;
-        setFieldDirty(FieldGeoLatitude);
-        setFieldDirty(FieldGeoLongitude);
-        updated();
-    }
-    // If hasGeo is true, the caller should set latitude and longitude to legal values..
-}
-#endif
 float Incidence::geoLatitude() const
 {
     Q_D(const Incidence);
@@ -1292,29 +1256,4 @@ void Incidence::deserialize(QDataStream &in)
     }
 }
 
-namespace
-{
-template<typename T>
-QVariantList toVariantList(int size, typename QVector<T>::ConstIterator begin, typename QVector<T>::ConstIterator end)
-{
-    QVariantList l;
-    l.reserve(size);
-    std::transform(begin, end, std::back_inserter(l), [](const T &val) {
-        return QVariant::fromValue(val);
-    });
-    return l;
-}
-
-} // namespace
-
-QVariantList Incidence::attachmentsVariant() const
-{
-    Q_D(const Incidence);
-    return toVariantList<Attachment>(d->mAttachments.size(), d->mAttachments.cbegin(), d->mAttachments.cend());
-}
-
-QVariantList Incidence::conferencesVariant() const
-{
-    Q_D(const Incidence);
-    return toVariantList<Conference>(d->mConferences.size(), d->mConferences.cbegin(), d->mConferences.cend());
-}
+#include "moc_incidence.cpp"
