@@ -23,9 +23,8 @@
 */
 #include "vcalformat.h"
 #include "calendar.h"
-#include "event.h"
+#include "calformat_p.h"
 #include "exceptions.h"
-#include "todo.h"
 
 #include "kcalendarcore_debug.h"
 
@@ -47,7 +46,7 @@ using namespace KCalendarCore;
 */
 //@cond PRIVATE
 template<typename K>
-void removeAllVCal(QVector<QSharedPointer<K>> &c, const QSharedPointer<K> &x)
+void removeAllVCal(QList<QSharedPointer<K>> &c, const QSharedPointer<K> &x)
 {
     if (c.count() < 1) {
         return;
@@ -63,7 +62,7 @@ void removeAllVCal(QVector<QSharedPointer<K>> &c, const QSharedPointer<K> &x)
     c.remove(c.indexOf(x));
 }
 
-class Q_DECL_HIDDEN KCalendarCore::VCalFormat::Private
+class KCalendarCore::VCalFormatPrivate : public CalFormatPrivate
 {
 public:
     Calendar::Ptr mCalendar;
@@ -74,24 +73,31 @@ public:
 //@endcond
 
 VCalFormat::VCalFormat()
-    : d(new KCalendarCore::VCalFormat::Private)
+    : CalFormat(new KCalendarCore::VCalFormatPrivate)
 {
 }
 
 VCalFormat::~VCalFormat()
 {
-    delete d;
+}
+
+static void mimeErrorHandler(char *e)
+{
+    qCWarning(KCALCORE_LOG) << "Error parsing vCalendar file:" << e;
 }
 
 bool VCalFormat::load(const Calendar::Ptr &calendar, const QString &fileName)
 {
+    Q_D(VCalFormat);
     d->mCalendar = calendar;
 
     clearException();
 
     // this is not necessarily only 1 vcal.  Could be many vcals, or include
     // a vcard...
+    registerMimeErrorHandler(&mimeErrorHandler);    // Note: vCalendar error handler provided by libical.
     VObject *vcal = Parse_MIME_FromFileName(const_cast<char *>(QFile::encodeName(fileName).data()));
+    registerMimeErrorHandler(nullptr);
 
     if (!vcal) {
         setException(new Exception(Exception::CalVersionUnknown));
@@ -102,7 +108,7 @@ bool VCalFormat::load(const Calendar::Ptr &calendar, const QString &fileName)
 
     // put all vobjects into their proper places
     auto savedTimeZoneId = d->mCalendar->timeZoneId();
-    populate(vcal, false, fileName);
+    populate(vcal);
     d->mCalendar->setTimeZoneId(savedTimeZoneId);
 
     // clean up from vcal API stuff
@@ -120,13 +126,9 @@ bool VCalFormat::save(const Calendar::Ptr &calendar, const QString &fileName)
     return false;
 }
 
-bool VCalFormat::fromString(const Calendar::Ptr &calendar, const QString &string, bool deleted, const QString &notebook)
-{
-    return fromRawString(calendar, string.toUtf8(), deleted, notebook);
-}
-
-bool VCalFormat::fromRawString(const Calendar::Ptr &calendar, const QByteArray &string, bool deleted, const QString &notebook)
+bool VCalFormat::fromRawString(const Calendar::Ptr &calendar, const QByteArray &string)
 {
+    Q_D(VCalFormat);
     d->mCalendar = calendar;
 
     if (!string.size()) {
@@ -143,7 +145,7 @@ bool VCalFormat::fromRawString(const Calendar::Ptr &calendar, const QByteArray &
 
     // put all vobjects into their proper places
     auto savedTimeZoneId = d->mCalendar->timeZoneId();
-    populate(vcal, deleted, notebook);
+    populate(vcal);
     d->mCalendar->setTimeZoneId(savedTimeZoneId);
 
     // clean up from vcal API stuff
@@ -153,11 +155,9 @@ bool VCalFormat::fromRawString(const Calendar::Ptr &calendar, const QByteArray &
     return true;
 }
 
-QString VCalFormat::toString(const Calendar::Ptr &calendar, const QString &notebook, bool deleted)
+QString VCalFormat::toString(const Calendar::Ptr &calendar)
 {
     Q_UNUSED(calendar);
-    Q_UNUSED(notebook);
-    Q_UNUSED(deleted);
 
     qCWarning(KCALCORE_LOG) << "Exporting into VCAL is not supported";
     return {};
@@ -165,6 +165,7 @@ QString VCalFormat::toString(const Calendar::Ptr &calendar, const QString &noteb
 
 Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
 {
+    Q_D(VCalFormat);
     VObject *vo = nullptr;
     VObjectIterator voi;
     char *s = nullptr;
@@ -323,12 +324,15 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
         uint recurrenceType = Recurrence::rNone;
         int recurrenceTypeAbbrLen = 0;
 
-        QString tmpStr = (QString::fromUtf8(s = fakeCString(vObjectUStringZValue(vo))));
+        s = fakeCString(vObjectUStringZValue(vo));
+        QString tmpStr = QString::fromUtf8(s);
         deleteStr(s);
         tmpStr = tmpStr.simplified();
         const int tmpStrLen = tmpStr.length();
         if (tmpStrLen > 0) {
             tmpStr = tmpStr.toUpper();
+            QStringView prefix = QStringView(tmpStr).left(2);
+
             // first, read the type of the recurrence
             recurrenceTypeAbbrLen = 1;
             if (tmpStr.at(0) == QLatin1Char('D')) {
@@ -337,13 +341,13 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
                 recurrenceType = Recurrence::rWeekly;
             } else if (tmpStrLen > 1) {
                 recurrenceTypeAbbrLen = 2;
-                if (tmpStr.leftRef(2) == QLatin1String("MP")) {
+                if (prefix == QLatin1String("MP")) {
                     recurrenceType = Recurrence::rMonthlyPos;
-                } else if (tmpStr.leftRef(2) == QLatin1String("MD")) {
+                } else if (prefix == QLatin1String("MD")) {
                     recurrenceType = Recurrence::rMonthlyDay;
-                } else if (tmpStr.leftRef(2) == QLatin1String("YM")) {
+                } else if (prefix == QLatin1String("YM")) {
                     recurrenceType = Recurrence::rYearlyMonth;
-                } else if (tmpStr.leftRef(2) == QLatin1String("YD")) {
+                } else if (prefix == QLatin1String("YD")) {
                     recurrenceType = Recurrence::rYearlyDay;
                 }
             }
@@ -353,7 +357,7 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
             // Immediately after the type is the frequency
             int index = tmpStr.indexOf(QLatin1Char(' '));
             int last = tmpStr.lastIndexOf(QLatin1Char(' ')) + 1; // find last entry
-            int rFreq = tmpStr.midRef(recurrenceTypeAbbrLen, (index - 1)).toInt();
+            const int rFreq = QStringView(tmpStr).mid(recurrenceTypeAbbrLen, (index - 1)).toInt();
             ++index; // advance to beginning of stuff after freq
 
             // Read the type-specific settings
@@ -487,8 +491,8 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
             index = last;
             if (tmpStr.mid(index, 1) == QLatin1String("#")) {
                 // Nr of occurrences
-                index++;
-                int rDuration = tmpStr.midRef(index, tmpStr.length() - index).toInt();
+                ++index;
+                const int rDuration = QStringView(tmpStr).mid(index, tmpStr.length() - index).toInt();
                 if (rDuration > 0) {
                     anEvent->recurrence()->setDuration(rDuration);
                 }
@@ -504,12 +508,11 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
     // recurrence exceptions
     if ((vo = isAPropertyOf(vtodo, VCExpDateProp)) != nullptr) {
         s = fakeCString(vObjectUStringZValue(vo));
-        QStringList exDates = QString::fromUtf8(s).split(QLatin1Char(','));
-        QStringList::ConstIterator it;
-        for (it = exDates.constBegin(); it != exDates.constEnd(); ++it) {
-            QDateTime exDate = ISOToQDateTime(*it);
+        const QStringList exDates = QString::fromUtf8(s).split(QLatin1Char(','));
+        for (const auto &date : exDates) {
+            const QDateTime exDate = ISOToQDateTime(date);
             if (exDate.time().hour() == 0 && exDate.time().minute() == 0 && exDate.time().second() == 0) {
-                anEvent->recurrence()->addExDate(ISOToQDate(*it));
+                anEvent->recurrence()->addExDate(ISOToQDate(date));
             } else {
                 anEvent->recurrence()->addExDateTime(exDate);
             }
@@ -621,6 +624,7 @@ Todo::Ptr VCalFormat::VTodoToEvent(VObject *vtodo)
 
 Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
 {
+    Q_D(VCalFormat);
     VObject *vo = nullptr;
     VObjectIterator voi;
     char *s = nullptr;
@@ -764,6 +768,8 @@ Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
         const int tmpStrLen = tmpStr.length();
         if (tmpStrLen > 0) {
             tmpStr = tmpStr.toUpper();
+            const QStringView prefix(tmpStr.left(2));
+
             // first, read the type of the recurrence
             recurrenceTypeAbbrLen = 1;
             if (tmpStr.at(0) == QLatin1Char('D')) {
@@ -772,13 +778,13 @@ Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
                 recurrenceType = Recurrence::rWeekly;
             } else if (tmpStrLen > 1) {
                 recurrenceTypeAbbrLen = 2;
-                if (tmpStr.leftRef(2) == QLatin1String("MP")) {
+                if (prefix == QLatin1String("MP")) {
                     recurrenceType = Recurrence::rMonthlyPos;
-                } else if (tmpStr.leftRef(2) == QLatin1String("MD")) {
+                } else if (prefix == QLatin1String("MD")) {
                     recurrenceType = Recurrence::rMonthlyDay;
-                } else if (tmpStr.leftRef(2) == QLatin1String("YM")) {
+                } else if (prefix == QLatin1String("YM")) {
                     recurrenceType = Recurrence::rYearlyMonth;
-                } else if (tmpStr.leftRef(2) == QLatin1String("YD")) {
+                } else if (prefix == QLatin1String("YD")) {
                     recurrenceType = Recurrence::rYearlyDay;
                 }
             }
@@ -788,7 +794,7 @@ Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
             // Immediately after the type is the frequency
             int index = tmpStr.indexOf(QLatin1Char(' '));
             int last = tmpStr.lastIndexOf(QLatin1Char(' ')) + 1; // find last entry
-            int rFreq = tmpStr.midRef(recurrenceTypeAbbrLen, (index - 1)).toInt();
+            const int rFreq = QStringView(tmpStr).mid(recurrenceTypeAbbrLen, (index - 1)).toInt();
             ++index; // advance to beginning of stuff after freq
 
             // Read the type-specific settings
@@ -922,8 +928,8 @@ Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
             index = last;
             if (tmpStr.mid(index, 1) == QLatin1String("#")) {
                 // Nr of occurrences
-                index++;
-                int rDuration = tmpStr.midRef(index, tmpStr.length() - index).toInt();
+                ++index;
+                const int rDuration = QStringView(tmpStr).mid(index, tmpStr.length() - index).toInt();
                 if (rDuration > 0) {
                     anEvent->recurrence()->setDuration(rDuration);
                 }
@@ -941,12 +947,11 @@ Event::Ptr VCalFormat::VEventToEvent(VObject *vevent)
     // recurrence exceptions
     if ((vo = isAPropertyOf(vevent, VCExpDateProp)) != nullptr) {
         s = fakeCString(vObjectUStringZValue(vo));
-        QStringList exDates = QString::fromUtf8(s).split(QLatin1Char(','));
-        QStringList::ConstIterator it;
-        for (it = exDates.constBegin(); it != exDates.constEnd(); ++it) {
-            QDateTime exDate = ISOToQDateTime(*it);
+        const QStringList exDates = QString::fromUtf8(s).split(QLatin1Char(','));
+        for (const auto &date : exDates) {
+            const QDateTime exDate = ISOToQDateTime(date);
             if (exDate.time().hour() == 0 && exDate.time().minute() == 0 && exDate.time().second() == 0) {
-                anEvent->recurrence()->addExDate(ISOToQDate(*it));
+                anEvent->recurrence()->addExDate(ISOToQDate(date));
             } else {
                 anEvent->recurrence()->addExDateTime(exDate);
             }
@@ -1186,6 +1191,7 @@ QString VCalFormat::qDateToISO(const QDate &qd)
 
 QString VCalFormat::qDateTimeToISO(const QDateTime &dt, bool zulu)
 {
+    Q_D(VCalFormat);
     if (!dt.isValid()) {
         return QString();
     }
@@ -1211,19 +1217,19 @@ QString VCalFormat::qDateTimeToISO(const QDateTime &dt, bool zulu)
 
 QDateTime VCalFormat::ISOToQDateTime(const QString &dtStr)
 {
-    QDate tmpDate;
-    QTime tmpTime;
-    QString tmpStr;
-    int year, month, day, hour, minute, second;
+    Q_D(VCalFormat);
+    auto noAllocString = QStringView{dtStr};
 
-    tmpStr = dtStr;
-    year = tmpStr.leftRef(4).toInt();
-    month = tmpStr.midRef(4, 2).toInt();
-    day = tmpStr.midRef(6, 2).toInt();
-    hour = tmpStr.midRef(9, 2).toInt();
-    minute = tmpStr.midRef(11, 2).toInt();
-    second = tmpStr.midRef(13, 2).toInt();
+    int year = noAllocString.left(4).toInt();
+    int month = noAllocString.mid(4, 2).toInt();
+    int day = noAllocString.mid(6, 2).toInt();
+    int hour = noAllocString.mid(9, 2).toInt();
+    int minute = noAllocString.mid(11, 2).toInt();
+    int second = noAllocString.mid(13, 2).toInt();
+
+    QDate tmpDate;
     tmpDate.setDate(year, month, day);
+    QTime tmpTime;
     tmpTime.setHMS(hour, minute, second);
 
     if (tmpDate.isValid() && tmpTime.isValid()) {
@@ -1233,18 +1239,18 @@ QDateTime VCalFormat::ISOToQDateTime(const QString &dtStr)
         } else {
             return QDateTime(tmpDate, tmpTime, d->mCalendar->timeZone());
         }
-    } else {
-        return QDateTime();
     }
+
+    return {};
 }
 
 QDate VCalFormat::ISOToQDate(const QString &dateStr)
 {
-    int year, month, day;
+    auto noAllocString = QStringView{dateStr};
 
-    year = dateStr.leftRef(4).toInt();
-    month = dateStr.midRef(4, 2).toInt();
-    day = dateStr.midRef(6, 2).toInt();
+    const int year = noAllocString.left(4).toInt();
+    const int month = noAllocString.mid(4, 2).toInt();
+    const int day = noAllocString.mid(6, 2).toInt();
 
     return QDate(year, month, day);
 }
@@ -1259,7 +1265,7 @@ bool VCalFormat::parseTZOffsetISO8601(const QString &s, int &result)
     // We also accept broken one without +
     int mod = 1;
     int v = 0;
-    QString str = s.trimmed();
+    const QString str = s.trimmed();
     int ofs = 0;
     result = 0;
 
@@ -1284,7 +1290,7 @@ bool VCalFormat::parseTZOffsetISO8601(const QString &s, int &result)
         return false;
     }
 
-    v = str.midRef(ofs, 2).toInt(&ok) * 60;
+    v = QStringView(str).mid(ofs, 2).toInt(&ok) * 60;
     if (!ok) {
         return false;
     }
@@ -1298,7 +1304,7 @@ bool VCalFormat::parseTZOffsetISO8601(const QString &s, int &result)
             if (str.size() < (ofs + 2)) {
                 return false;
             }
-            v += str.midRef(ofs, 2).toInt(&ok);
+            v += QStringView(str).mid(ofs, 2).toInt(&ok);
             if (!ok) {
                 return false;
             }
@@ -1311,9 +1317,9 @@ bool VCalFormat::parseTZOffsetISO8601(const QString &s, int &result)
 // take a raw vcalendar (i.e. from a file on disk, clipboard, etc. etc.
 // and break it down from it's tree-like format into the dictionary format
 // that is used internally in the VCalFormat.
-void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
+void VCalFormat::populate(VObject *vcal)
 {
-    Q_UNUSED(notebook);
+    Q_D(VCalFormat);
     // this function will populate the caldict dictionary and other event
     // lists. It turns vevents into Events and then inserts them.
 
@@ -1364,7 +1370,6 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
         QStringList tzList;
         QString tz;
         int utcOffset;
-        int utcOffsetDst;
         if (parseTZOffsetISO8601(ts, utcOffset)) {
             // qCDebug(KCALCORE_LOG) << "got standard offset" << ts << utcOffset;
             // standard from tz
@@ -1393,6 +1398,7 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
                         continue;
                     }
 
+                    int utcOffsetDst;
                     if (parseTZOffsetISO8601(argl[1], utcOffsetDst)) {
                         // qCDebug(KCALCORE_LOG) << "got DST offset" << argl[1] << utcOffsetDst;
                         // standard
@@ -1473,21 +1479,11 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
                     !anEvent->hasRecurrenceId() ? d->mCalendar->event(anEvent->uid()) : d->mCalendar->event(anEvent->uid(), anEvent->recurrenceId());
 
                 if (old) {
-                    if (deleted) {
-                        d->mCalendar->deleteEvent(old); // move old to deleted
-                        removeAllVCal(d->mEventsRelate, old);
-                    } else if (anEvent->revision() > old->revision()) {
+                    if (anEvent->revision() > old->revision()) {
                         d->mCalendar->deleteEvent(old); // move old to deleted
                         removeAllVCal(d->mEventsRelate, old);
                         d->mCalendar->addEvent(anEvent); // and replace it with this one
                     }
-                } else if (deleted) {
-                    old = !anEvent->hasRecurrenceId() ? d->mCalendar->deletedEvent(anEvent->uid())
-                                                      : d->mCalendar->deletedEvent(anEvent->uid(), anEvent->recurrenceId());
-                    if (!old) {
-                        d->mCalendar->addEvent(anEvent); // add this one
-                        d->mCalendar->deleteEvent(anEvent); // and move it to deleted
-                    }
                 } else {
                     d->mCalendar->addEvent(anEvent); // just add this one
                 }
@@ -1498,7 +1494,7 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
                 if (hasTimeZone && !aTodo->allDay() && aTodo->dtStart().timeZone() == QTimeZone::utc()) {
                     // This sounds stupid but is how others are doing it, so here
                     // we go. If there is a TZ in the VCALENDAR even if the dtStart
-                    // and dtend are in UTC, clients interpret it usint alse the TZ defined
+                    // and dtend are in UTC, clients interpret it using also the TZ defined
                     // in the Calendar. I know it sounds braindead but oh well
                     int utcOffSet = aTodo->dtStart().offsetFromUtc();
                     QDateTime dtStart(aTodo->dtStart().addSecs(utcOffSet));
@@ -1512,20 +1508,11 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
                 }
                 Todo::Ptr old = !aTodo->hasRecurrenceId() ? d->mCalendar->todo(aTodo->uid()) : d->mCalendar->todo(aTodo->uid(), aTodo->recurrenceId());
                 if (old) {
-                    if (deleted) {
-                        d->mCalendar->deleteTodo(old); // move old to deleted
-                        removeAllVCal(d->mTodosRelate, old);
-                    } else if (aTodo->revision() > old->revision()) {
+                    if (aTodo->revision() > old->revision()) {
                         d->mCalendar->deleteTodo(old); // move old to deleted
                         removeAllVCal(d->mTodosRelate, old);
                         d->mCalendar->addTodo(aTodo); // and replace it with this one
                     }
-                } else if (deleted) {
-                    old = d->mCalendar->deletedTodo(aTodo->uid(), aTodo->recurrenceId());
-                    if (!old) {
-                        d->mCalendar->addTodo(aTodo); // add this one
-                        d->mCalendar->deleteTodo(aTodo); // and move it to deleted
-                    }
                 } else {
                     d->mCalendar->addTodo(aTodo); // just add this one
                 }
@@ -1545,13 +1532,12 @@ void VCalFormat::populate(VObject *vcal, bool deleted, const QString &notebook)
     } // while
 
     // Post-Process list of events with relations, put Event objects in relation
-    Event::List::ConstIterator eIt;
-    for (eIt = d->mEventsRelate.constBegin(); eIt != d->mEventsRelate.constEnd(); ++eIt) {
-        (*eIt)->setRelatedTo((*eIt)->relatedTo());
+    for (const auto &eventPtr : std::as_const(d->mEventsRelate)) {
+        eventPtr->setRelatedTo(eventPtr->relatedTo());
     }
-    Todo::List::ConstIterator tIt;
-    for (tIt = d->mTodosRelate.constBegin(); tIt != d->mTodosRelate.constEnd(); ++tIt) {
-        (*tIt)->setRelatedTo((*tIt)->relatedTo());
+
+    for (const auto &todoPtr : std::as_const(d->mTodosRelate)) {
+        todoPtr->setRelatedTo(todoPtr->relatedTo());
     }
 
     // Now lets put the TZ back as it was if we have changed it.
@@ -1661,19 +1647,14 @@ void VCalFormat::readCustomProperties(VObject *o, const Incidence::Ptr &i)
 
 void VCalFormat::writeCustomProperties(VObject *o, const Incidence::Ptr &i)
 {
+    Q_D(VCalFormat);
     const QMap<QByteArray, QString> custom = i->customProperties();
-    for (QMap<QByteArray, QString>::ConstIterator c = custom.begin(); c != custom.end(); ++c) {
-        if (d->mManuallyWrittenExtensionFields.contains(c.key()) || c.key().startsWith("X-KDE-VOLATILE")) { // krazy:exclude=strings
+    for (auto cIt = custom.cbegin(); cIt != custom.cend(); ++cIt) {
+        const QByteArray property = cIt.key();
+        if (d->mManuallyWrittenExtensionFields.contains(property) || property.startsWith("X-KDE-VOLATILE")) { // krazy:exclude=strings
             continue;
         }
 
-        addPropValue(o, c.key().constData(), c.value().toUtf8().constData());
+        addPropValue(o, property.constData(), cIt.value().toUtf8().constData());
     }
 }
-
-void VCalFormat::virtual_hook(int id, void *data)
-{
-    Q_UNUSED(id);
-    Q_UNUSED(data);
-    Q_ASSERT(false);
-}
